package com.malk.service.aliwork;

import com.malk.server.aliwork.YDParam;

import java.util.List;

/**
 * 函数式编程 & 异步并发实现
 */
public interface YDService {

    /**
     * 自定义回调方式
     * -
     * 非异步情况下，和网络请求一致，回调默认同步等待
     */
    enum INVOKE_TYPE {
        AWAIT_CONCURRENCE, // 并发累积回调 accumulation
        ASYNC_CONCURRENCE, // 并发异步执行
        SYNC_SINGLE, // 同步单次回调
        SYNC_BATCH,  // 同步累积回调 accumulation
    }

    /**
     * 设置AWAIT_CONCURRENCE模式下, 超时上限返回已成功数据
     * -
     * 因YDClient内错误为抛出形式, 且目前全量异步等待通过while实现, 若出现异常则会导致死循环, 异步情况下不会执行相应的回调 [包含Lambda与Future]
     */
    Integer AWAIT_TERMINATION = 600000;

    /**
     * 设置AWAIT_CONCURRENCE模式下, while循环下休眠时间
     */
    Integer AWAIT_INTERVAL = 400;

    /**
     * 查询一个切片单位 UPPER_LIMIT 的全部数据 [同步:单次回调/同步:累积回调/并发:异步执行/并发:累积回调]
     */
    void queryLimitData(String api, YDParam param, INVOKE_TYPE type, QueryLambda lambda);

    /**
     * 全量查询, 函数操作, api定义返回data或者是id的集合 [符合条件表单数据 ==> 三个步骤]
     */
    void queryAllData(String api, YDParam param, INVOKE_TYPE type, QueryLambda lambda);

    /**
     * 全量查询, 同步累积 [返回data]
     */
    List queryAllData(YDParam param);

    /**
     * 全量查询, 并发累积 [返回data]
     */
    List queryAllDataByConcurrence(YDParam param);

    /**
     * 批量操作, 函数逻辑 [兼容更新和删除, 取决于param.updateFormDataJson]
     */
    void batchDelOrUpt(YDParam param, List<String> idList, INVOKE_TYPE type, OperateLambda lambda);

    /**
     * 批量操作, 一个分页 [查询分页, 同步删除]
     */
    String operatePageData(YDParam param);

    /**
     * 批量操作, 一个切片 [同步查询, 异步删除]
     */
    String operateLimitData(YDParam param);

    /**
     * 批量操作, 全量数量 [并发累积, 异步删除]
     */
    String operateAllData(YDParam param);

    /**
     * 函数式编程：Stream类、Lambda表达式和函数接口（Functional Inteface）
     * -
     * 自定义Lambda是为了更好的代码提示和数据类型定义: @FunctionalInterface，该接口为函数接口，一个函数接口只能存在一个方法
     * 特别注意，非异步情况下，和网络请求一致，回调默认同步等待【回调方式可自定义：如兼容并发可有，同步单次回调、同步累积回调、并发异步执行、并发累积回调】
     * 函数编程的最大好处，是可传入执行逻辑，而无需等待全部数据结果，尤其并发和异步情况下【异步效果优于Future】
     * -
     * 结合异步与并发, @Async说明和配置详见core/AsyncConfig
     * -
     * 错误抛出与拦截详见 CatchException
     * 关于入参为JSONString类型: body内的json需要传入字符串, 若为对象会解析报错. 若是formData, 传入对象会自动转为字符串, 不会解析错误, 无需转为JSONString
     */
    @FunctionalInterface
    interface QueryLambda {
        void dataList(List list);
    }

    @FunctionalInterface
    interface OperateLambda {
        void accumulation(Integer count, Integer total);
    }
}