package com.malk.service.aliwork.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.xxpt.gateway.shared.client.http.ExecutableClient;
import com.alibaba.xxpt.gateway.shared.client.http.PostClient;
import com.malk.repository.dao.mutual.McAuthorizationDao;
import com.malk.repository.entity.mutual.McAuthorizationDTO;
import com.malk.server.aliwork.YDConf;
import com.malk.server.aliwork.YDParam;
import com.malk.server.aliwork.YDR;
import com.malk.server.common.McException;
import com.malk.service.aliwork.YDClient;
import lombok.Synchronized;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.AsyncResult;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Future;

/**
 * 表单接口适用于流程: 查询和更新以及删除
 */
@Service
@Slf4j
public class YDClientImpl implements YDClient {

    private McAuthorizationDao useAuthDao;

    // 官方推荐, 建议构造器注入代替字段注入
    @Autowired
    private YDClientImpl(McAuthorizationDao useAuthDao) {
        this.useAuthDao = useAuthDao;
    }

    /**
     * 获取秘钥: 开放目的为批量操作无需每次请求都取值数据库
     */
    @Synchronized
    @Override
    public void loadSystemToken(YDParam param) {
        // 避免每次请求都查询数据库秘钥
        if (StringUtils.isNotBlank(param.getSystemToken())) return;
        McAuthorizationDTO useAuthDo = useAuthDao.findByAppType(param.getAppType());
        log.info("获取宜搭授权秘钥, {}", useAuthDo);
        if (ObjectUtil.isNull(useAuthDo)) {
            McException.exceptionAuth("应用授权未添加");
        }
        param.setSystemToken(useAuthDo.getSystemToken());
    }

    // 创建请求: 查询为data之集合, 新增为实例Id, 删除/更新为空, ....
    private YDR createRequest(String api, YDParam param) {
        if (StringUtils.isBlank(param.getSystemToken())) {
            loadSystemToken(param);
        }
        PostClient postClient = ExecutableClient.getInstance().newPostClient(api);
        Map data = JSONObject.parseObject(JSON.toJSONString(param), Map.class);
        log.info("宜搭请求入参, param = {}, api = {}", data, api);
        for (Object key : data.keySet()) {
            // fastjson已屏蔽掉值为null字段 [宜搭null, 会直接报错]
            String field = String.valueOf(key);
            postClient.addParameter(field, String.valueOf(data.get(field)));
        }
        String rsp = postClient.post();
        log.debug("响应数据, {}", rsp);
        return JSON.parseObject(rsp, YDR.class);
    }

    /**
     * 查询数据
     */
    @Override
    public Map queryData(String api, YDParam param) {
        YDR response = createRequest(api, param);
        if (Boolean.TRUE.equals(response.isSuccess())) {
            if (api.equals(YDConf.API_OPEN_URL)) {
                Map rsp = new HashMap();
                rsp.put("data", response.getResult());
                return rsp;
            }
            Map result = (Map) response.getResult();
            log.info("宜搭查询返回, currentPage = {}, dataSize = {}, totalCount = {}", param.getCurrentPage(), ((List) result.get("data")).size(), result.get("totalCount"));
            return result;
        }
        throw new McException(response.getErrorCode(), response.getErrorMsg());
    }

    /**
     * 其它操作
     */
    @Override
    public Object operateData(String api, YDParam param) {
        YDR response = createRequest(api, param);
        if (Boolean.TRUE.equals(response.isSuccess())) {
            log.info("宜搭操作返回, {}", response);
            return response.getResult();
        }
        throw new McException(response.getErrorCode(), response.getErrorMsg());
    }

    /**
     * 并发线程实现: 查询 [统一配置调用上限避免超过并发]
     */
    @Async("aliworkConcurrence")
    @Override
    public Future queryDataByConcurrence(String api, YDParam param) {
        return new AsyncResult(queryData(api, param));
    }

    /**
     * 并发线程函数: 查询 [统一配置调用上限避免超过并发]
     */
    @Async("aliworkConcurrence")
    @Override
    public void queryDataByConcurrence(String api, YDParam param, QueryLambda lambda) {
        lambda.dataMap(queryData(api, param));
    }

    /**
     * 并发线程实现: 操作 [统一配置调用上限避免超过并发]
     */
    @Async("aliworkConcurrence")
    @Override
    public Future operateDataByConcurrence(String api, YDParam param) {
        return new AsyncResult(operateData(api, param));
    }

    /**
     * 并发线程函数: 操作 [统一配置调用上限避免超过并发]
     */
    @Async("aliworkConcurrence")
    @Override
    public void operateDataByConcurrence(String api, YDParam param, OperateLambda lambda) {
        lambda.result(operateData(api, param));
    }
}
