package com.malk.service.aliwork.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONObject;
import com.malk.server.aliwork.YDConf;
import com.malk.server.aliwork.YDParam;
import com.malk.server.common.McException;
import com.malk.server.common.McREnum;
import com.malk.service.aliwork.YDClient;
import com.malk.service.aliwork.YDService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.concurrent.atomic.AtomicReference;

/**
 * * 服务请求_宜搭
 */
@Service
@Slf4j
public class YDServiceImpl implements YDService {

    private YDClient ydClient;

    // 官方推荐, 建议构造器注入代替字段注入
    @Autowired
    public YDServiceImpl(YDClient ydClient) {
        this.ydClient = ydClient;
    }

    // 查询序列号范围: 接口查询, 默认是降序
    private List<String> queryFormSerialArea(String api, YDParam ydParam) {
        String compIdSerial = ydParam.getCompIdSerial();
        if (StringUtils.isBlank(compIdSerial)) {
            McException.exceptionParam("compIdSerial不能为空");
        }
        // 对象拷贝: 避免排序条件被修改
        YDParam param = ObjectUtil.clone(ydParam);
        param.setPageSize(1);
        List serialArea = new LinkedList();
        param.setDynamicOrder("+", compIdSerial);
        Map response = ydClient.queryData(api, param);
        List<Map> resplist = (List) response.get("data");
        McException.assertException(resplist.size() == 0, McREnum.IGNORE_EXECUTE);
        serialArea.add(((Map) (resplist.get(0).get("formData"))).get(compIdSerial + "_value"));
        param.setDynamicOrder("-", compIdSerial);
        response = ydClient.queryData(api, param);
        resplist = (List) response.get("data");
        serialArea.add(((Map) (resplist.get(0).get("formData"))).get(compIdSerial + "_value"));
        serialArea.add(response.get("totalCount").toString());
        log.info("本次全量范围, start, end, total = {}", serialArea);
        return serialArea;
    }

    /**
     * 查询一个切片单位 UPPER_LIMIT 的全部数据
     * -
     * 函数 [同步:单次回调/同步:累积回调/并发:异步执行/并发:累积回调]
     */
    @Override
    public void queryLimitData(String api, YDParam param, INVOKE_TYPE type, QueryLambda lambda) {
        ydClient.loadSystemToken(param); // 避免每次请求都查询数据库秘钥
        param.setPageSize(1);
        Map response = ydClient.queryData(api, param);
        Integer totalCount = (Integer) response.get("totalCount");
        if (totalCount == 0) return; // 兼容全量, 不能throw, 序列号范围可能存在空
        if (totalCount > YDConf.UPPER_LIMIT) {
            McException.exceptionExecute("单次查询超过宜搭实现, 请调用全量方法");
        }
        log.info("查询一个切片单位, totalCount = {}", totalCount);
        // 承载累积全量的集合数据回调
        List dataList = new ArrayList(); // 动态数组: 对于随机访问get和set，ArrayList觉得优于LinkedList，因为LinkedList要移动指针
        List<Future> futureList = new LinkedList<>(); // 链表结构: 对于新增和删除操作，LinedList比较占优势，因为ArrayList要移动数据
        // Math.ceil 需要有浮点数, 否则转 int 类型忽略掉, 向上取整就会是当前值
        for (int index = 0; index < (int) Math.ceil(totalCount * 1.0 / YDConf.PAGE_SIZE_LIMIT); index++) {
            // 对象拷贝: 避免并发下分页条件被修改导致数据混乱
            YDParam ydParam = (YDParam) param.copyParam();
            ydParam.setCurrentPage(index + 1);
            ydParam.setPageSize(YDConf.PAGE_SIZE_LIMIT);
            switch (type) {
                // 同步单次回调
                case SYNC_SINGLE:
                    lambda.dataList((List) ydClient.queryData(api, ydParam).get("data"));
                    break;
                // 并发异步执行
                case ASYNC_CONCURRENCE:
                    ydClient.queryDataByConcurrence(api, ydParam, data -> lambda.dataList((List) data.get("data")));
                    break;
                // 并发累积回调
                case AWAIT_CONCURRENCE:
                    // NOTE: AWAIT_CONCURRENCE模式下, 要使用Future, 回调函数没有状态, 若执行过程中出现数据增删, 简单的通过数量判断不可取
                    futureList.add(ydClient.queryDataByConcurrence(api, ydParam));
                    break;
                // 同步累积回调
                case SYNC_BATCH:
                    dataList.addAll((List) ydClient.queryData(api, ydParam).get("data"));
                    break;
                default:
                    break;
            }
        }
        if (INVOKE_TYPE.SYNC_BATCH.equals(type)) {
            lambda.dataList(dataList);
            log.info("并发累积一个切片单位, accumulative = {}, totalCount = {}", dataList.size(), totalCount);
        }
        if (INVOKE_TYPE.AWAIT_CONCURRENCE.equals(type)) {
            // NOTE: 设置AWAIT_CONCURRENCE模式下, 超时上限返回已成功数据 [并发等待与throw解决]
            Integer accumulateTimestamp = 0;
            while (true) {
                // 降低cpu性能消耗
                try {
                    Thread.sleep(AWAIT_INTERVAL);
                } catch (InterruptedException e) {
                    log.error(e.getMessage(), e); // 记录错误日志
                }
                accumulateTimestamp += AWAIT_INTERVAL;
                // 跳出等待, 回调全部数据
                if (futureList.isEmpty() || accumulateTimestamp >= AWAIT_TERMINATION) {
                    lambda.dataList(dataList);
                    log.info("并发累积一个切片单位, accumulative = {}, totalCount = {}", dataList.size(), totalCount);
                    break;
                }
                // 普通for循环移除元素不会抛出异常
                for (int i = 0; i < futureList.size(); i++) {
                    Future future = futureList.get(i);
                    if (future.isDone()) {
                        // 判断线程结束，获取回调信息，并将该回调清除: 需要捕获两种异常
                        try {
                            dataList.addAll(((List) ((Map) future.get()).get("data")));
                            futureList.remove(future);
                        } catch (InterruptedException e) {
                            log.error(e.getMessage(), e); // 记录错误日志
                        } catch (ExecutionException e) {
                            log.error(e.getMessage(), e); // 记录错误日志
                        }
                    } else {
                        continue;
                    }
                }
            }
        }
    }

    /**
     * 全量查询: 符合条件表单数据 ==> 三个步骤
     * 1. 页面新建一个数值组件, 写入值类型流水号, 作为切片查询之依据
     * -    a. 组件必须为数值作为范围查询条件, 尤其注意流水号重置规则不要溢出最大位数: 虽然超出宜搭也会存储, 但查询范围会无限扩大
     * -    b. 若流水号需要前缀使用公式执行拼接, 若带有文本宜搭也会存入, 但排序和范围查询会失效
     * -    [流水号使用纯数字, 前缀需要设置为1: 为0范围查询会异常, 不能使用年月日: 会导致切片进入死循环, 因为年月日在流水号高位]
     * 2. 查询页面需要为表单, 流程为业务需求, 表单为数据环节, 状态回写
     * -    [流程不支持排序, 需要查询符合条件之第一条到最后一条流水号作为范围]
     * 3. 全量接口实现
     * -    a. 依照流水号降序查询一条数据取值开始范围, 再升序查询一条数据取值结束范围
     * -    b. 再以 5k/3w 作为查询范围, 确保数据查询不会溢出
     * -    c. 最后再每一个范围内由 totalCount 按照 pageSize 100 循环同步数据
     * 4. 异步并发通过控制配置线程池 [详见AsyncConfig]
     * 5. 函数式编程: 特别注意，非异步情况下，和网络请求一致，回调默认同步等待. 若需要等待执行结果可通过Future控制线程回调
     */
    @Override
    public void queryAllData(String api, YDParam param, INVOKE_TYPE type, QueryLambda lambda) {
        ydClient.loadSystemToken(param); // 避免每次请求都查询数据库秘钥
        // NOTE: 尝试try一个limit, 在catch内再执行全量: 放弃原因, 性能提升有限, 且都有ExceptionCom的抛出, 处理过于麻烦且不符合方法唯一性
        List<String> serialArea = queryFormSerialArea(api, param);
        Long start = Long.parseLong(serialArea.get(0));
        Long end = Long.parseLong(serialArea.get(1));
        Long total = Long.parseLong(serialArea.get(2));
        // 服务累积集合
        List dataList = new ArrayList();
        // 保持单次和全量返回逻辑
        boolean isAccumulative = INVOKE_TYPE.SYNC_BATCH.equals(type) || INVOKE_TYPE.AWAIT_CONCURRENCE.equals(type);
        Map searchJson = new HashMap();
        if (StringUtils.isNotBlank(param.getSearchFieldJson())) {
            searchJson.putAll(JSONObject.parseObject(param.getSearchFieldJson(), Map.class));
        }
        // Math.ceil 需要有浮点数, 否则转 int 类型忽略掉, 向上取整就会是当前值
        for (int index = 0; index <= (int) Math.floor((end - start) * 1.0 / YDConf.UPPER_LIMIT); index++) {
            Long begin = start + index * YDConf.UPPER_LIMIT;
            Long finish = begin + YDConf.UPPER_LIMIT - 1;
            if (finish > end) finish = end;
            List sectionArea = new ArrayList<>();
            sectionArea.add(begin);
            sectionArea.add(finish);
            searchJson.put(param.getCompIdSerial(), sectionArea);
            param.setSearchFieldJson(JSONObject.toJSONString(searchJson));
            if (isAccumulative) {
                queryLimitData(api, param, type, dataList::addAll);
            } else {
                queryLimitData(api, param, type, lambda);
            }
        }
        if (isAccumulative) {
            lambda.dataList(dataList);
            log.info("本次全量返回, accumulative = {}, totalCount = {}", dataList.size(), total);
        }

    }

    /**
     * 全量查询, 同步累积
     */
    @Override
    public List queryAllData(YDParam param) {
        List dataList = new ArrayList();
        queryAllData(YDConf.API_FORM_QUERY_DATA, param, INVOKE_TYPE.SYNC_BATCH, dataList::addAll);
        return dataList;
    }

    /**
     * 全量查询, 并发累积
     */
    @Override
    public List queryAllDataByConcurrence(YDParam param) {
        List dataList = new ArrayList();
        queryAllData(YDConf.API_FORM_QUERY_DATA, param, INVOKE_TYPE.AWAIT_CONCURRENCE, dataList::addAll);
        return dataList;
    }

    /**
     * 批量操作: 兼容更新和删除, 取决于param.updateFormDataJson
     * -
     * 函数式编程逻辑与全量查询一致, 针对每次查询的结果分页内每一条进行操作, 回调颗粒度更细 [操作回调单次为每一条数据, 查询单次回调为每一页数据]
     */
    @Override
    public void batchDelOrUpt(YDParam param, List<String> idList, INVOKE_TYPE type, OperateLambda lambda) {
        ydClient.loadSystemToken(param); // 避免每次请求都查询数据库秘钥
        boolean isUpdate = StringUtils.isNotBlank(param.getUpdateFormDataJson());
        String API = YDConf.API_FORM_DELETE;
        if (isUpdate) API = YDConf.API_FORM_UPDATE;
        McException.assertException(idList.size() == 0, McREnum.IGNORE_EXECUTE);
        // 承载累积操作数量回调
        Integer count = 0;
        Integer total = idList.size();
        log.info("合计执行数量, {}", total);
        for (String formInstId : idList) {
            YDParam ydParam = ObjectUtil.clone(param);
            ydParam.setFormInstId(formInstId);
            switch (type) {
                // 同步单次回调
                case SYNC_SINGLE:
                    ydClient.operateData(API, ydParam);
                    lambda.accumulation(1, total);
                    break;
                // 并发异步执行
                case ASYNC_CONCURRENCE:
                    ydClient.operateDataByConcurrence(API, ydParam, result -> lambda.accumulation(1, total));
                    break;
                // 并发累积回调
                case AWAIT_CONCURRENCE:
                    ydClient.operateDataByConcurrence(API, ydParam);
                    count += 1;
                    break;
                // 同步累积回调
                case SYNC_BATCH:
                    ydClient.operateData(API, ydParam);
                    count += 1;
                    break;
                default:
                    break;
            }
        }
        if (INVOKE_TYPE.SYNC_BATCH.equals(type)) {
            lambda.accumulation(count, total);
            log.info("累积执行: {}, 执行成功: {}", total, count);
        }
        if (INVOKE_TYPE.AWAIT_CONCURRENCE.equals(type)) {
            // NOTE: 设置AWAIT_CONCURRENCE模式下, 超时上限返回已成功数据
            Integer accumulateTimestamp = 0;
            // 操作颗粒度最细, 不能使用sleep
            while (true) {
                // 跳出等待, 回调执行数量
                if (count == total) {
                    lambda.accumulation(count, total);
                    log.info("累积执行: {}, 执行数量: {}", total, count);
                    break;
                }
            }
        }
    }

    /**
     * 操作一个分页数据 [查询分页, 同步删除]
     */
    @Override
    public String operatePageData(YDParam param) {
        List<String> idList = (List) ydClient.queryData(YDConf.API_FORM_QUERY_ID, param).get("data");
        AtomicReference<Integer> accumulationCount = new AtomicReference<>(0);
        AtomicReference<Integer> totalCount = new AtomicReference<>(0);
        batchDelOrUpt(param, idList, INVOKE_TYPE.SYNC_BATCH, (count, total) -> {
            accumulationCount.updateAndGet(v -> v + count);
            totalCount.updateAndGet(v -> v + total);
        });
        return "合计执行数量: " + totalCount + ", 执行成功数量: " + accumulationCount;
    }

    /**
     * 操作宜搭一个查询上限累计数据 [同步查询, 异步删除: 错误就抛出中断]
     */
    @Override
    public String operateLimitData(YDParam param) {
        AtomicReference<Integer> accumulationCount = new AtomicReference<>(0);
        queryLimitData(YDConf.API_FORM_QUERY_ID, param, INVOKE_TYPE.SYNC_BATCH, list -> {
            accumulationCount.updateAndGet(v -> v + list.size());
            batchDelOrUpt(param, list, INVOKE_TYPE.ASYNC_CONCURRENCE, (count, total) -> {
            });
        });
        return "合计执行数量: " + accumulationCount;
    }

    /**
     * 操作宜搭全量查询数据 [并发累积, 异步删除: 错误就抛出中断]
     */
    @Override
    public String operateAllData(YDParam param) {
        AtomicReference<Integer> accumulationCount = new AtomicReference<>(0);
        queryAllData(YDConf.API_FORM_QUERY_ID, param, INVOKE_TYPE.AWAIT_CONCURRENCE, list -> {
            accumulationCount.updateAndGet(v -> v + list.size());
            batchDelOrUpt(param, list, INVOKE_TYPE.ASYNC_CONCURRENCE, (count, total) -> {
            });
        });
        return "合计执行数量: " + accumulationCount;
    }
}
