package com.malk.service.dingtalk.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.dingtalk.api.request.OapiProcessinstanceCreateRequest;
import com.malk.Util.UtilHttp;
import com.malk.Util.UtilList;
import com.malk.Util.UtilToken;
import com.malk.server.dingtalk.DDApproveDTO;
import com.malk.server.dingtalk.DDConf;
import com.malk.server.dingtalk.DDR;
import com.malk.service.dingtalk.DDClient;
import lombok.Synchronized;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.validation.constraints.NotNull;
import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Service
public class DDClientImpl implements DDClient {

    @Autowired
    private DDConf ddConf;

    // token授权参数: 旧版本
    private Map initTokenParams() {
        Map param = new HashMap();
        param.put("access_token", getAccessToken());
        return param;
    }

    /**
     * 获取访问授权
     *
     * @apiNote https://open.dingtalk.com/document/personalapp-server/obtain-user-token
     */
    @Synchronized
    @Override
    public String getAccessToken() {
        String accessToken = UtilToken.get("invalid-token-dingtalk");
        if (StringUtils.isNotBlank(accessToken)) return accessToken;
        Map param = new HashMap();
        param.put("appkey", ddConf.getAppKey());
        param.put("appsecret", ddConf.getAppSecret());
        DDR r = (DDR) UtilHttp.doGet("https://oapi.dingtalk.com/gettoken", param, DDR.class);
        log.info("响应token, {}", r.getAccessToken());
        accessToken = r.getAccessToken();
        // token失效自动重置: DD重新调用会重置过期时间
        UtilToken.put("invalid-token-dingtalk", accessToken, r.getExpiresIn() * 1000L);
        return accessToken;
    }

    /**
     * 获取指定用户的所有父部门列表
     *
     * @apiNote https://open.dingtalk.com/document/orgapp-server/queries-the-list-of-all-parent-departments-of-a-user
     */
    @Override
    public Map listParentByUser(String userId) {
        Map param = new HashMap();
        param.put("access_token", getAccessToken());
        Map body = new HashMap();
        body.put("userid", userId);
        DDR r = (DDR) UtilHttp.doPost("https://oapi.dingtalk.com/topapi/v2/department/listparentbyuser", param, body, DDR.class);
        return (Map) r.getResult();
    }

    /**
     * 获取部门详情
     *
     * @apiNote https://open.dingtalk.com/document/isvapp-server/query-department-details0-v2
     */
    @Override
    public Map departmentDetail(Number deptId) {
        JSONObject body = new JSONObject();
        body.put("dept_id", deptId);
        DDR r = (DDR) UtilHttp.doPost("https://oapi.dingtalk.com/topapi/v2/department/get", initTokenParams(), body, DDR.class);
        return (Map) r.getResult();
    }

    /**
     * 查询用户详情
     *
     * @apiNote https://open.dingtalk.com/document/orgapp-server/query-user-details
     */
    @Override
    public Map getUserInfo(String userId) {
        JSONObject body = new JSONObject();
        body.put("userid", userId);
        DDR r = (DDR) UtilHttp.doPost("https://oapi.dingtalk.com/topapi/v2/user/get", null, initTokenParams(), body, DDR.class);
        return (Map) r.getResult();
    }

    /**
     * 发起审批实例
     *
     * @apiNote https://open.dingtalk.com/document/orgapp-server/initiate-approval
     */
    @Override
    public String createProcessInstance(DDApproveDTO dto) {
        Map body = new HashMap();
        body.put("process_code", dto.getProcessCode());
        body.put("originator_user_id", dto.getOriginatorUserId());
        body.put("dept_id", dto.getDeptId());
        // 审批流表单参数，设置各表单项值
        List<OapiProcessinstanceCreateRequest.FormComponentValueVo> formComponentValues = new ArrayList<>();
        for (DDApproveDTO.FormComponentValue form : dto.getFormValues()) {
            // 主表组件
            OapiProcessinstanceCreateRequest.FormComponentValueVo formComponentValueVo = new OapiProcessinstanceCreateRequest.FormComponentValueVo();
            formComponentValueVo.setName(form.getName());
            if (ObjectUtil.isNotNull(form.getDetails())) {
                // 明细组件: 每一个组件都是集合, table为集合嵌套集合
                List<List<OapiProcessinstanceCreateRequest.FormComponentValueVo>> formComponentTable = new ArrayList<>();
                for (List<DDApproveDTO.FormComponentValue> table : form.getDetails()) {
                    List<OapiProcessinstanceCreateRequest.FormComponentValueVo> formComponentDetails = new ArrayList<>();
                    for (DDApproveDTO.FormComponentValue detail : table) {
                        OapiProcessinstanceCreateRequest.FormComponentValueVo item = new OapiProcessinstanceCreateRequest.FormComponentValueVo();
                        item.setName(detail.getName());
                        item.setValue(detail.getValue());
                        formComponentDetails.add(item);
                    }
                    formComponentTable.add(formComponentDetails);
                }
                formComponentValueVo.setValue(JSON.toJSONString(formComponentTable));
            } else {
                formComponentValueVo.setValue(form.getValue());
            }
            formComponentValues.add(formComponentValueVo);
        }
        body.put("form_component_values", formComponentValues);
        DDR r = (DDR) UtilHttp.doPost("https://oapi.dingtalk.com/topapi/processinstance/create", initTokenParams(), body, DDR.class);
        return r.getProcessInstanceId();
    }

    /**
     * 获取审批实例详情
     *
     * @apiNote https://open.dingtalk.com/document/orgapp-server/obtains-the-details-of-a-single-approval-instance
     */
    @Override
    public Map queryProcessInstanceId(String processInstanceId) {
        Map body = new HashMap();
        body.put("process_instance_id", processInstanceId);
        DDR r = (DDR) UtilHttp.doPost("https://oapi.dingtalk.com/topapi/processinstance/get", null, initTokenParams(), body, DDR.class);
        return r.getProcessInstance();
    }

    /**
     * 撤销审批实例
     *
     * @apiNote https://open.dingtalk.com/document/orgapp-server/terminate-a-workflow-by-using-an-instance-id
     */
    @Override
    public boolean terminateRunningApprove(String processInstanceId, boolean isSystem, String remark, String operatingUserId) {
        Map request = new HashMap();
        request.put("process_instance_id", processInstanceId);
        request.put("is_system", isSystem);
        request.put("remark", remark);
        request.put("operating_userid", operatingUserId);
        Map body = new HashMap();
        body.put("request", request);
        DDR r = (DDR) UtilHttp.doPost("https://oapi.dingtalk.com/topapi/process/instance/terminate", null, initTokenParams(), body, DDR.class);
        return (Boolean) r.getResult();
    }

    /**
     * 通知审批撤销
     *
     * @apiNote https://open.dingtalk.com/document/orgapp/notify-the-attendance-to-modify-the-punch-result-when-the
     */
    @Override
    public boolean cancelRunningApprove(String userid, String approve_id) {
        Map body = new HashMap();
        body.put("approve_id", approve_id);
        body.put("userid", userid);
        DDR r = (DDR) UtilHttp.doPost("https://oapi.dingtalk.com/topapi/attendance/approve/cancel", null, initTokenParams(), body, DDR.class);
        return (Boolean) r.getResult();
    }

    /**
     * 添加审批评论
     *
     * @apiNote https://open.dingtalk.com/document/orgapp-server/add-an-approval-comment
     */
    @Override
    public boolean commentApproveInstance(@NotNull String processInstanceId, File file, @NotNull String text, @NotNull String commentUserid) {
        Map body = new HashMap();
        body.put("process_instance_id", processInstanceId);
        body.put("file", file);
        body.put("text", text);
        body.put("comment_userid", commentUserid);
        Map req = new HashMap();
        req.put("request", body);
        DDR r = (DDR) UtilHttp.doPost("https://oapi.dingtalk.com/topapi/process/instance/comment/add", initTokenParams(), req, DDR.class);
        return (Boolean) r.getResult();
    }

    /**
     * 获取推送失败的事件列表: 推送失败钉钉记录, 获取后记录会被清空
     *
     * @apiNote https://open.dingtalk.com/document/orgapp-server/obtain-the-event-list-of-failed-push-messages
     */
    @Override
    public List<Map> queryFailedList() {
        DDR r = (DDR) UtilHttp.doGet("https://oapi.dingtalk.com/topapi/processinstance/get", initTokenParams(), DDR.class);
        return r.getFailedList();
    }

    /**
     * 发送工作通知
     *
     * @apiNote https://open.dingtalk.com/document/isvapp-server/asynchronous-sending-of-enterprise-session-messages
     */
    @Override
    public void sendCorpConversationMessage(@NotNull String agentId, List<String> useridList, List<String> deptIdList, boolean toAllUser, @NotNull JSONObject message) {
        Map body = new HashMap();
        body.put("agent_id", agentId);
        if (UtilList.isNotEmpty(useridList)) body.put("userid_list", String.join(",", useridList));
        if (UtilList.isNotEmpty(deptIdList)) body.put("dept_id_list", String.join(",", deptIdList));
        body.put("to_all_user", toAllUser);
        body.put("msg", message);
        UtilHttp.doPost("https://oapi.dingtalk.com/topapi/message/corpconversation/asyncsend_v2", null, initTokenParams(), body, DDR.class);
    }

    /**
     * 查询假期规则列表
     *
     * @apiNote https://open.dingtalk.com/document/orgapp-server/holiday-type-query?spm=ding_open_doc.document.0.0.24e942a0FbC2xl#topic-2023081
     */
    @Override
    public List<Map> queryVacationTypeList(String op_userid, boolean vacation_source) {
        Map body = new HashMap();
        body.put("op_userid", op_userid);
        body.put("vacation_source", Boolean.TRUE.equals(vacation_source) ? "all" : "");
        DDR ddr = (DDR) UtilHttp.doPost("https://oapi.dingtalk.com/topapi/attendance/vacation/type/list", null, initTokenParams(), body, DDR.class);
        return (List<Map>) ddr.getResult();
    }

    /**
     * 查询假期余额 [分页是根据游标控制]
     *
     * @apiNote https://open.dingtalk.com/document/isvapp-server/query-holiday-balance
     */
    @Override
    public List<Map> queryVacationQuota(String op_userid, String leave_code, String userids, int offset, int size) {
        Map body = new HashMap();
        body.put("leave_code", leave_code);
        body.put("op_userid", op_userid);
        body.put("userids", userids);
        body.put("offset", offset);
        body.put("size", size);
        DDR ddr = (DDR) UtilHttp.doPost("https://oapi.dingtalk.com/topapi/attendance/vacation/quota/list", null, initTokenParams(), body, DDR.class);
        List<Map> records = (List<Map>) ((Map) ddr.getResult()).get("leave_quotas");
        if (Boolean.TRUE.equals(((Map<?, ?>) ddr.getResult()).get("has_more"))) {
            records.addAll(queryVacationQuota(op_userid, leave_code, userids, offset + size, size));
        }
        return records;
    }
}