package com.malk.service.diwork.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.core.type.TypeReference;
import com.malk.Util.UtilToken;
import com.malk.server.common.McException;
import com.malk.server.common.McREnum;
import com.malk.server.diwork.YSConf;
import com.malk.server.diwork.accessToken.AccessTokenResponse;
import com.malk.server.diwork.accessToken.GenericResponse;
import com.malk.server.diwork.accessToken.RequestTool;
import com.malk.server.diwork.accessToken.SignHelper;
import com.malk.service.diwork.YSClient;
import lombok.SneakyThrows;
import lombok.Synchronized;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.validation.constraints.NotNull;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Consumer;

@Service
@Slf4j
public class YSClientImpl implements YSClient {

    @Autowired
    private YSConf ysConf;

    // 断言错误信息
    private static void assertSuccess(GenericResponse rsp) {
        McException.assertException(!rsp.isSuccess() && !rsp.isSuccessToken(), rsp.getCode(), rsp.getMessage(), "YS");
    }

    // 获取访问授权
    @Synchronized
    @Override
    public String getAccessToken() {
        String accessToken = UtilToken.get("invalid-token-yonsuite");
        if (StringUtils.isNotBlank(accessToken)) return accessToken;
        Map<String, String> params = new HashMap<>();
        // 除签名外的其他参数
        params.put("appKey", ysConf.getAppKey());
        params.put("timestamp", String.valueOf(System.currentTimeMillis()));
        // 计算签名
        String signature = null;
        try {
            signature = SignHelper.sign(params, ysConf.getAppSecret());
        } catch (NoSuchAlgorithmException e) {
            log.error(e.getMessage(), e); // 记录错误日志
        } catch (UnsupportedEncodingException e) {
            log.error(e.getMessage(), e); // 记录错误日志
        } catch (InvalidKeyException e) {
            log.error(e.getMessage(), e); // 记录错误日志
        }
        params.put("signature", signature);
        // 请求
        String requestUrl = "https://api.diwork.com/open-auth/selfAppAuth/getAccessToken";
        GenericResponse<AccessTokenResponse> response = null;
        try {
            response = RequestTool.doGet(requestUrl, params, new TypeReference<GenericResponse<AccessTokenResponse>>() {
            });
        } catch (IOException e) {
            log.error(e.getMessage(), e); // 记录错误日志
        }
        assertSuccess(response);
        log.info("获取YS授权, {}", response.getData());
        accessToken = response.getData().getAccessToken();
        // token失效自动重置: YS重新调用不会重置过期时间
        UtilToken.put("invalid-token-yonsuite", accessToken, response.getData().getExpire() * 1000L);
        return accessToken;
    }

    // 执行查询
    private List<Map> executeRequest(String url, String condition, QueryLambda lambda) {
        log.info("查询入参, {}", condition);
        GenericResponse response = JSON.parseObject(HttpUtil.post(url, condition), GenericResponse.class);
        assertSuccess(response);
        Integer totalCount = (Integer) ((Map) response.getData()).get("totalCount");
        List dataList = (List<Map>) ((Map) response.getData()).get("info");
        log.info("响应数量: 总数量 {}, 当前页 {}", totalCount, dataList.size());
        lambda.dataList(dataList, totalCount);
        return dataList;
    }

    // 执行操作
    private void executeOperation(String url, String condition, Consumer lambda) {
        log.info("请求入参, {}", condition);
        GenericResponse response = JSON.parseObject(HttpUtil.post(url, condition), GenericResponse.class);
        assertSuccess(response);
        log.info("请求响应, {}", response.toString());
        lambda.accept(response.isSuccess());
    }

    // 查询订单公共参数
    private JSONObject initQueryOrderParam() {
        JSONObject param = new JSONObject();
        param.put("headselectfields", "id, buyer_nick, receiver_name, isbizaudited, isfinaudited, cTradeCode, cShopID, pay_time, buyer_message, seller_memo, trade_memo, receiver_mobile, receiver_address");
        param.put("bodyselectfields", "id, productID, price, num, total_fee, payment");
        param.put("shipdetailselectfields", "id");
        // param.put("shopcode", shopCode); // 不传查询所有
        param.put("isclosed", 0);
        return param;
    }

    /**
     * B2C订单查询
     */
    @Override
    public void queryTradeOrder(String accessToken, int pageIndex, String updateStart, String updateEnd, QueryLambda lambda) {
        JSONObject param = initQueryOrderParam();
        param.put("pageIndex", pageIndex);
        param.put("start_pubts", updateStart);
        param.put("end_pubts", updateEnd);
        param.put("pageSize", YSConf.PAGE_SIZE_LIMIT);
        // param.put("tradecode", "DD999920220605000002");
        String url = "https://api.diwork.com/yonbip/sd/dst/tradeorder/query?access_token=" + accessToken;
        JSONObject condition = new JSONObject();
        condition.put("partParam", param);
        executeRequest(url, condition.toJSONString(), lambda);
    }

    /**
     * B2C订单查询交易编号\订单ids
     */
    @Override
    public List<Map> queryTradeOrderByCondition(String tradeCode, String ids) {
        JSONObject param = initQueryOrderParam();
        param.put("pageIndex", 1);
        if (ObjectUtil.isNotNull(ids)) {
            param.put("pageSize", ysConf.PAGE_SIZE_LIMIT);
            param.put("ids", ids);
        } else {
            param.put("pageSize", 1);
            param.put("tradecode", tradeCode);
        }
        String url = "https://api.diwork.com/yonbip/sd/dst/tradeorder/query?access_token=" + getAccessToken();
        JSONObject condition = new JSONObject();
        condition.put("partParam", param);
        List<Map> dataList = executeRequest(url, condition.toJSONString(), (list, total) -> {

        });
        McException.assertException(dataList.isEmpty(), McREnum.VALIDATED_PARAM.getCode(), "b2c条件查询订单返回为空", "YS");
        return dataList;
    }

    /**
     * 店铺信息查询
     */
    @Override
    public void queryStoreArchives(@NotNull(message = "token不能为空") String accessToken, QueryLambda lambda) {
        JSONObject param = new JSONObject();
        param.put("stopstatus", 0);
        param.put("pageIndex", 1);
        param.put("pageSize", YSConf.PAGE_SIZE_LIMIT);
        String url = "https://api.diwork.com/yonbip/sd/dst/storearchives/query?access_token=" + accessToken;
        JSONObject condition = new JSONObject();
        condition.put("partParam", param);
        executeRequest(url, condition.toJSONString(), lambda);
    }

    /**
     * 订单财审
     */
    @Override
    public void financialApprove(String accessToken, String orderIds, String payTime, Consumer lambda) {
        JSONObject param = new JSONObject();
        param.put("ids", orderIds);
        JSONObject reflect = new JSONObject();
        reflect.put(orderIds, payTime); // 多个可能要拆为集合
        param.put("partParam", reflect);
        String url = "https://api.diwork.com/yonbip/sd/dst/tradeorder/finapprove?access_token=" + accessToken;
        executeOperation(url, param.toJSONString(), lambda);
    }

    /**
     * 订单弃审
     */
    @Override
    public void abandonApprove(String accessToken, String orderIds, String payTime, String reason, Consumer lambda) {
        JSONObject param = new JSONObject();
        param.put("ids", orderIds);
        JSONObject reflect = new JSONObject();
        reflect.put(orderIds, payTime); // 多个可能要拆为集合
        param.put("partParam", reflect);
        JSONObject approve = new JSONObject();
        if (StringUtils.isBlank(reason)) reason = "拒绝";
        approve.put("cUnApproveReason", reason);
        param.put("externalData", approve);
        String url = "https://api.diwork.com/yonbip/sd/dst/tradeorder/unapprove?access_token=" + accessToken;
        executeOperation(url, param.toJSONString(), lambda);
    }

    /**
     * 订单弃审[新] - 日期不影响, 传入是订单id, 不是交易编号/订单编号
     */
    @Override
    public void abandonApprove(String ids, String reason) {
        JSONObject param = new JSONObject();
        param.put("ids", ids);
        JSONObject reflect = new JSONObject();
        for (String id : ids.split(", ")) {
            reflect.put(id, new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()));
        }
        param.put("partParam", reflect);
        JSONObject approve = new JSONObject();
        approve.put("cUnApproveReason", reason);
        param.put("externalData", approve);
        String url = "https://api.diwork.com/yonbip/sd/dst/tradeorder/unapprove?access_token=" + getAccessToken();
        executeOperation(url, param.toJSONString(), obj -> {

        });
    }

    /**
     * 物料档案查询
     */
    @Synchronized
    @SneakyThrows
    @Override
    public Map getProductDetail(String accessToken, String productId) {
        // NOTE 接口限流1秒1次. ppExt 需要添加 @Synchronized, 避免异步情况下执行触发限流
        Thread.sleep(1000);
        log.info("物料查询, {}", productId);
        Map response = null;
        // NOTE: HttpUtil 请求报错 Request method must be one of POST, PUT, PATCH or DELETE to decode a multipart request
        // String url = "https://yonbip.diwork.com/iuap-api-gateway/yonbip/digitalModel/product/detail?access_token=" + accessToken + "&orgId=001&id=" + productId;
        // Map response = JSON.parseObject(HttpUtil.get(url));
        try {
            Map param = new HashMap();
            param.put("access_token", accessToken);
            param.put("orgId", "001");
            param.put("id", productId);
            response = JSONObject.parseObject(RequestTool.doGet("https://yonbip.diwork.com/iuap-api-gateway/yonbip/digitalModel/product/detail", param), Map.class);
        } catch (IOException e) {
            log.error(e.getMessage(), e); // 记录错误日志
        }
        Map data = (Map) response.get("data");
        float fPrimeCosts = Float.valueOf(String.valueOf(((Map) data.get("detail")).get("fPrimeCosts")));
        float fMarkPrice = Float.valueOf(String.valueOf(((Map) data.get("detail")).get("fMarkPrice")));
        Map result = new HashMap();
        result.put("fPrimeCosts", fPrimeCosts);
        result.put("fMarkPrice", fMarkPrice);
        log.info("响应数据, 采购价 = {}, 零售价 = {}", fPrimeCosts, fMarkPrice);
        return result;
    }

    /**
     * 获取店铺系数
     *
     * @return 店铺系数: 最低价 = 店铺数据 * 采购最低价(最低单价)
     */
    @Override
    public Map<String, Float> getShopCoefficient() {
        Map shop = new HashMap();
        queryStoreArchives(getAccessToken(), (list, total) -> {
            for (Map map : list) {
                float coefficient = 1.0f;
                String shopCode = String.valueOf(map.get("shopCode"));
                if (shopCode.contains("-")) {
                    String[] arrStr = shopCode.split("-");
                    coefficient = Float.valueOf(arrStr[arrStr.length - 1]);
                }
                shop.put(shopCode, coefficient);
            }
        });
        log.info("店铺系数, {}", shop);
        return shop;
    }
}
