package com.malk.service.feishu.impl;

import cn.hutool.core.util.CharsetUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.malk.Util.UtilHttp;
import com.malk.Util.UtilList;
import com.malk.Util.UtilToken;
import com.malk.server.feishu.FSApproveDTO;
import com.malk.server.feishu.FSConf;
import com.malk.server.feishu.FSR;
import com.malk.service.feishu.FSClient;
import lombok.Synchronized;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.validation.constraints.NotNull;
import java.io.File;
import java.util.*;
import java.util.function.Consumer;

@Slf4j
@Service
public class FSClientImpl implements FSClient {

    @Autowired
    private FSConf fsConf;

    // 获取访问授权
    @Synchronized
    private String getAccessToken() {
        String accessToken = UtilToken.get("invalid-token-feishu");
        if (StringUtils.isNotBlank(accessToken)) return accessToken;
        Map param = new HashMap();
        param.put("app_id", fsConf.getAppKey());
        param.put("app_secret", fsConf.getAppSecret());
        FSR r = createPOSTRequest("/open-apis/auth/v3/app_access_token/internal", param, null);
        log.info("响应token, {}", r.getTenant_access_token());
        accessToken = r.getTenant_access_token();
        // token失效自动重置: FS重新调用不会重置过期时间
        UtilToken.put("invalid-token-feishu", accessToken, r.getExpire() * 1000L);
        return accessToken;
    }

    // 创建请求实例: 加载token
    private FSR createGETRequest(String url, Map param) {
        return createGETRequest(url, param, true);
    }

    // 创建GET请求: 获取token
    private FSR createGETRequest(String url, Map param, boolean needToken) {
        HashMap<String, String> headers = new HashMap<>();
        if (needToken) {
            headers.put("Authorization", "Bearer " + getAccessToken());
        }
        if (!url.contains("http")) {
            url = "https://open.feishu.cn" + url;
        }
        log.info("请求入参, {}, {}", url, param);
        String rsp = HttpUtil.createGet(url).addHeaders(headers).form(param).execute().body();
        FSR r = JSON.parseObject(rsp, FSR.class);
        log.info("响应数据, {}", r.getCode());
        r.assertSuccess();
        return r;
    }

    // 创建POST请求
    private FSR createPOSTRequest(String url, Map<String, Object> param, String body) {
        HashMap<String, String> headers = new HashMap<>();
        headers.put("Authorization", "Bearer " + getAccessToken());
        if (!url.contains("http")) {
            url = "https://open.feishu.cn" + url;
        }
        log.info("请求入参, {}, {}, {}", url, param, body);
        url = HttpUtil.urlWithForm(url, param, CharsetUtil.CHARSET_UTF_8, true);
        // NOTE: 1. 注意 .form 是表单内容; .body 才是请求内容 [Restful请求] 2. 使用转码, 避免中文字符导致参数识别异常, 如中文小括号
        String rsp = HttpUtil.createPost(url).addHeaders(headers).body(body).execute().body();
        FSR r = JSON.parseObject(rsp, FSR.class);
        log.info("响应状态: {}", r.getCode());
        r.assertSuccess();
        return r;
    }

    /**
     * 查询班表信息: 用户是否排班 [日期格式为yyyy-MM-dd]
     */
    @Override
    public List<Map> queryAttendanceForUserDailyShifts(String checkDateFrom, String checkDateTo, String... userIds) {
        Map param = new HashMap();
        param.put("employee_type", "employee_id");
        JSONObject body = new JSONObject();
        body.put("check_date_from", checkDateFrom);
        body.put("check_date_to", checkDateTo);
        body.put("user_ids", Arrays.asList(userIds));
        FSR r = createPOSTRequest("/open-apis/attendance/v1/user_daily_shifts/query", param, body.toJSONString());
        return (List<Map>) ((Map) r.getData()).get("user_daily_shifts");
    }

    /**
     * code2session 免登[网页]
     */
    @Override
    public Map codeToAuthorization(String authCode) {
        JSONObject body = new JSONObject();
        body.put("code", authCode);
        body.put("grant_type", "authorization_code");
        FSR r = createPOSTRequest("/open-apis/authen/v1/access_token", null, body.toJSONString());
        return (Map) r.getData();
    }

    /**
     * code2session 免登[小程序]
     */
    @Override
    public Map codeToSession(String authCode) {
        JSONObject body = new JSONObject();
        body.put("code", authCode);
        FSR r = createPOSTRequest("/open-apis/mina/v2/tokenLoginValidate", null, body.toJSONString());
        return (Map) r.getData();
    }

    /**
     * 获取单个用户信息
     */
    @Override
    public Map queryUserById(String userId) {
        Map param = new HashMap();
        param.put("user_id_type", "user_id");
        return (Map) createGETRequest("/open-apis/contact/v3/users/" + userId, param).getData();
    }

    /**
     * 创建审批实例
     */
    @Override
    public String createProcessInstance(FSApproveDTO approveDTO) {
        FSR fsr = createPOSTRequest("https://www.feishu.cn/approval/openapi/v2/instance/create", null, JSONObject.toJSONString(approveDTO));
        return String.valueOf(((Map) fsr.getData()).get("instance_code"));
    }

    /**
     * 获取单个审批实例详情
     */
    @Override
    public Map queryProcessInstanceById(String processInstanceId) {
        Map param = new HashMap();
        param.put("instance_code", processInstanceId);
        FSR fsr = createGETRequest("https://www.feishu.cn/approval/openapi/v2/instance/get", param);
        return (Map) fsr.getData();
    }

    /**
     * 量查询打卡流水记录 [日期格式为秒级时间戳]
     */
    @Override
    public List<Map> queryAttendanceFlows(String checkTimeFrom, String checkTimeTo, String... userIds) {
        Map param = new HashMap();
        param.put("employee_type", "employee_id");
        JSONObject body = new JSONObject();
        body.put("check_time_from", checkTimeFrom);
        body.put("check_time_to", checkTimeTo);
        body.put("user_ids", Arrays.asList(userIds));
        FSR r = createPOSTRequest("/open-apis/attendance/v1/user_flows/query", param, body.toJSONString());
        Map rsp = ((Map) r.getData());
        if (ObjectUtil.isNull(rsp.get("user_flow_results"))) {
            return new ArrayList<>();
        }
        log.info("查看考勤数据, {}", rsp.get("user_flow_results"));
        return (List<Map>) rsp.get("user_flow_results");
    }

    /**
     * 批量查询打卡流水记录
     */
    @Override
    public Map<String, List<Map>> queryAttendanceFlowsAssembleByUserId(String checkTimeFrom, String checkTimeTo, String... userIds) {
        List<Map> records = queryAttendanceFlows(checkTimeFrom, checkTimeTo, userIds);
        // 支持多userId查询, 组装返回格式为 user: List, 为定时任务降低查询频次
        Map<String, List<Map>> response = new HashMap();
        for (Map record : records) {
            String userId = String.valueOf(record.get("user_id"));
            if (ObjectUtil.isNull(response.get(userId))) {
                response.put(userId, new ArrayList<>());
            }
            response.get(userId).add(record);
        }
        return response;
    }

    /**
     * 查看审批模板定义
     */
    @Override
    public Map getProcessTemplateDefine(String processCode) {
        Map param = new HashMap();
        param.put("approval_code", processCode);
        return (Map) createGETRequest("https://www.feishu.cn/approval/openapi/v2/approval/get", param).getData();
    }

    /**
     * 订阅审批模板
     */
    @Override
    public void subscribeProcessCode(String processCode) {
        Map param = new HashMap();
        param.put("approval_code", processCode);
        createGETRequest("https://www.feishu.cn/approval/openapi/v2/subscription/subscribe", param);
    }

    /**
     * 取消订阅审批模板
     */
    @Override
    public void unsubscribeProcessCode(String processCode) {
        Map param = new HashMap();
        param.put("approval_code", processCode);
        createGETRequest("https://www.feishu.cn/approval/openapi/v2/subscription/unsubscribe", param);
    }

    /**
     * 审批结果写入: 将三果数据回写到飞书的考勤系统中
     *
     * @apiNote https://open.feishu.cn/document/uAjLw4CM/ukTMukTMukTM/reference/attendance-v1/user_approval/add-approved-data-in-feishu-attendance2
     */
    @Override
    public Map createOverApproveState(String userId, String date, String prop, JSONArray content) {
        Map param = new HashMap();
        param.put("employee_type", "employee_id");
        JSONObject approval = new JSONObject();
        approval.put("user_id", userId);
        approval.put("date", date);
        approval.put(prop, content);
        JSONObject body = new JSONObject();
        body.put("user_approval", approval);
        return (Map) createPOSTRequest("/open-apis/attendance/v1/user_approvals", param, body.toJSONString()).getData();
    }

    /**
     * 通知审批状态更新
     *
     * @apiNote https://open.feishu.cn/document/uAjLw4CM/ukTMukTMukTM/Attendance//task/notify-approval-status-update2
     */
    @Override
    public Map updateProcessState(String approvalId, String approvalType, int status) {
        Map param = new HashMap();
        param.put("employee_type", "employee_id");
        JSONObject body = new JSONObject();
        body.put("approval_id", approvalId);
        body.put("approval_type", approvalType);
        body.put("status", status);
        return (Map) createPOSTRequest("/open-apis/attendance/v1/approval_infos/process", param, body.toJSONString()).getData();
    }

    /**
     * 获取部门信息列表
     *
     * @param pageToken    分页标记，第一次请求不填，表示从头开始遍历；分页查询结果还有更多项时会同时返回新的 page_token，下次遍历可采用该 page_token 获取查询结果
     * @param departmentId 父部门的ID，填上获取部门下所有子部门，此处填写的 ID 必须是 department_id_type 指定的 ID
     * @param fetchChild   是否递归获取子部门
     * @param lambda       自定义: 是否累计获取, 通过 lambda 可异步接收 data, 若为 null 则同步返回 map
     * @return { page_token, has_more, items }
     * @apiNote https://open.feishu.cn/document/uAjLw4CM/ukTMukTMukTM/reference/contact-v3/department/list
     */
    @Override
    public Map queryDepartmentList(String pageToken, String departmentId, boolean fetchChild, Consumer lambda) {
        Map param = new HashMap();
        param.put("user_id_type", "user_id");
        param.put("department_id_type", "department_id");
        param.put("fetch_child", fetchChild);
        param.put("page_token", pageToken);
        param.put("page_size", FSConf.PAGE_SIZE_LIMIT);
        param.put("parent_department_id", departmentId);
        Map data = (Map) createGETRequest("/open-apis/contact/v3/departments", param).getData();
        if (ObjectUtil.isNull(lambda)) {
            return data;
        }
        lambda.accept(ObjectUtil.isNull(data.get("items")) ? new ArrayList<>() : data.get("items"));
        // 异步回调, 通过 lambda 递归查询
        if (Boolean.TRUE.equals(data.get("has_more"))) {
            queryDepartmentList(String.valueOf(data.get("page_token")), departmentId, fetchChild, lambda);
        }
        return null;
    }

    /**
     * 获取部门直属用户列表
     *
     * @param pageToken    分页标记，第一次请求不填，表示从头开始遍历；分页查询结果还有更多项时会同时返回新的 page_token，下次遍历可采用该 page_token 获取查询结果
     * @param departmentId 填写该字段表示获取该部门下用户，必填。根部门的部门ID为 0
     * @return { page_token, has_more, items }
     * @apiNote https://open.feishu.cn/document/uAjLw4CM/ukTMukTMukTM/reference/contact-v3/user/find_by_department
     */
    @Override
    public Map queryUserListByDepartmentId(String pageToken, String departmentId, Consumer lambda) {
        Map param = new HashMap();
        param.put("user_id_type", "user_id");
        param.put("department_id_type", "department_id");
        param.put("department_id", departmentId);
        param.put("page_token", pageToken);
        param.put("page_size", FSConf.PAGE_SIZE_LIMIT);
        Map data = (Map) createGETRequest("/open-apis/contact/v3/users/find_by_department", param).getData();
        if (ObjectUtil.isNull(lambda)) {
            return data;
        }
        lambda.accept(ObjectUtil.isNull(data.get("items")) ? new ArrayList<>() : data.get("items"));
        // 异步回调, 通过 lambda 递归查询
        if (Boolean.TRUE.equals(data.get("has_more"))) {
            queryUserListByDepartmentId(String.valueOf(data.get("page_token")), departmentId, lambda);
        }
        return null;
    }


    /// ppExt: 新版本访问方式
    private Map initHeaderToken() {
        HashMap<String, String> headers = new HashMap<>();
        headers.put("Authorization", "Bearer " + getAccessToken());
        return headers;
    }

    /**
     * 审批单据: 上传文件 [飞书不支持直接传地址] ppExt 1. 存储本地后拿到文件对象上传飞书, 2. 飞书为后台即可, 前端虽然可调用但会有白名单限制
     *
     * @apiNote https://open.feishu.cn/document/ukTMukTMukTM/uUDOyUjL1gjM14SN4ITN
     */
    @Override
    public Map uploadFileForApprove(String name, String type, File content) {
        Map form = new HashMap();
        form.put("name", name);
        form.put("type", type);
        form.put("content", content);
        FSR r = (FSR) UtilHttp.doUpload("https://www.feishu.cn/approval/openapi/v2/file/upload", initHeaderToken(), form, FSR.class);
        log.info("响应状态: {}", r.getCode());
        return (Map) r.getData();
    }

    /**
     * 查询统计数据 [日期格式为yyyyMMdd]
     *
     * @apiNote https://open.feishu.cn/document/uAjLw4CM/ukTMukTMukTM/reference/attendance-v1/user_stats_data/query
     */
    @Override
    public List<Map> queryUserAttendanceStatisticsData(@NotNull String statsType, @NotNull int startDate, @NotNull int endDate, List<String> userIds, boolean needHistory, boolean currentGroupOnly) {
        Map param = new HashMap();
        param.put("employee_type", "employee_id");
        Map body = new HashMap();
        body.put("locale", "zh");
        body.put("stats_type", statsType);
        body.put("start_date", startDate);
        body.put("end_date", endDate);
        if (UtilList.isNotEmpty(userIds)) body.put("user_ids", userIds);
        body.put("need_history", needHistory);
        body.put("current_group_only", currentGroupOnly);
        FSR r = (FSR) UtilHttp.doPost("https://open.feishu.cn/open-apis/attendance/v1/user_stats_datas/query", initHeaderToken(), param, body, FSR.class);
        log.info("响应状态: {}", r.getCode());
        return (List<Map>) ((Map) r.getData()).get("user_datas");
    }

    /**
     * 查询统计表头
     */
    @Override
    public Map<String, ?> queryUserAttendanceStatisticsHeader(@NotNull String statsType, @NotNull int startDate, @NotNull int endDate) {
        Map param = new HashMap();
        param.put("employee_type", "employee_id");
        Map body = new HashMap();
        body.put("locale", "zh");
        body.put("stats_type", statsType);
        body.put("start_date", startDate);
        body.put("end_date", endDate);
        FSR r = (FSR) UtilHttp.doPost("https://open.feishu.cn/open-apis/attendance/v1/user_stats_fields/query", initHeaderToken(), param, body, FSR.class);
        log.info("响应状态: {}", r.getCode());
        return (Map<String, ?>) ((Map) r.getData()).get("user_stats_field");
    }
}
