package com.malk.service.u8open.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.CharsetUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.malk.Util.UtilToken;
import com.malk.server.u8open.U8Conf;
import com.malk.server.u8open.U8R;
import com.malk.service.u8open.U8Client;
import lombok.Synchronized;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
@Slf4j
public class U8ClientImpl implements U8Client {

    @Autowired
    private U8Conf u8Conf;

    // 初始化参数: 获取token
    private Map<String, String> initParam() {
        Map param = new HashMap();
        param.put("from_account", u8Conf.getFromAccount());
        param.put("to_account", u8Conf.getToAccount());
        param.put("app_key", u8Conf.getAppKey());
        return param;
    }

    // 初始化参数: 加载token
    private Map<String, String> initParamNeedToken() {
        Map param = initParam();
        param.put("token", getAccessToken());
        return param;
    }

    // 获取访问授权
    @Synchronized
    private String getAccessToken() {
        String accessToken = UtilToken.get("invalid-token-u8");
        if (StringUtils.isNotBlank(accessToken)) return accessToken;
        Map param = initParam();
        param.put("app_secret", u8Conf.getAppSecret());
        Map tokenInfo = JSON.parseObject(createGETRequest("/system/token", param).getToken());
        log.info("响应数据, {}", tokenInfo);
        accessToken = String.valueOf(tokenInfo.get("id"));
        // token失效自动重置: U8重新调用会重置过期时间
        Integer expire = Integer.valueOf(String.valueOf(tokenInfo.get("expiresIn")));
        UtilToken.put("invalid-token-feishu", accessToken, expire * 1000L);
        return accessToken;
    }

    // 创建GET请求
    private U8R createGETRequest(String path, Map param) {
        log.info("请求入参, {}, {}", path, param);
        U8R r = JSON.parseObject(HttpUtil.get("https://api.yonyouup.com" + path, param), U8R.class);
        r.assertSuccess();
        return r;
    }

    // 创建POST请求
    private U8R createPOSTRequest(String path, Map<String, Object> param, String body) {
        log.info("请求入参, {}, {}, {}", path, param, body);
        // NOTE: 使用转码, 避免中文字符导致参数识别异常, 如中文小括号
        String url = HttpUtil.urlWithForm("https://api.yonyouup.com" + path, param, CharsetUtil.CHARSET_UTF_8, true);
        U8R r = JSON.parseObject(HttpUtil.post(url, body), U8R.class);
        r.assertSuccess();
        return r;
    }

    /**
     * 获取采购订单列表
     */
    @Override
    public List<Map> queryPurchaseOrderList(String dateBegin, String dateEnd) {
        Map param = initParamNeedToken();
        param.put("page_index", 1);
        param.put("rows_per_page", U8Conf.PAGE_SIZE_LIMIT);
        param.put("state", "审核中");
        param.put("date_begin", dateBegin);
        param.put("date_end", dateEnd);
        U8R r = createGETRequest("/api/purchaseorderlist/batch_get", param);
        if (ObjectUtil.isNull(r.getPurchaseorderlist())) r.setPurchaseorderlist(new ArrayList<>());
        log.info("响应数量: 总数量 {}, 当前页 {}", r.getRow_count(), r.getPurchaseorderlist().size());
        return r.getPurchaseorderlist();
    }

    /**
     * 获取单张采购订单
     */
    @Override
    public Map queryPurchaseOrderDetail(String voucherCode) {
        Map param = initParamNeedToken();
        param.put("id", voucherCode);
        U8R r = createGETRequest("/api/purchaseorder/get", param);
        log.info("响应数据: {}", r.getPurchaseorder());
        return r.getPurchaseorder();
    }

    /**
     * 平台交易号
     */
    @Override
    public String queryTradeId() {
        Map param = initParamNeedToken();
        param.remove("to_account");
        U8R r = createGETRequest("/system/tradeid", param);
        log.info("响应数据: {}", r.getTrade());
        return String.valueOf(r.getTrade().get("id"));
    }

    /**
     * 审核/弃审一张采购订单
     */
    @Override
    public void approvePurchaseOrder(String voucherCode, boolean isVerify) {
        String path = "/api/purchaseorder/unverify";
        if (isVerify) {
            path = "/api/purchaseorder/verify";
        }
        Map param = initParamNeedToken();
        param.put("tradeid", voucherCode);
        JSONObject voucher = new JSONObject();
        voucher.put("voucher_code", voucherCode);
        JSONObject purchase = new JSONObject();
        purchase.put("purchaseorder", voucher);
        U8R r = createPOSTRequest(path, param, purchase.toJSONString());
        log.info("响应数据: {}", r);
    }

    /**
     * 审核/弃审采购订单: 需审批人
     */
    @Override
    public void approvePurchaseOrderByPerson(String voucherCode, String personCode, boolean isAgree, String remark) {
        Map param = initParamNeedToken();
        param.put("tradeid", voucherCode);
        JSONObject voucher = new JSONObject();
        voucher.put("voucher_code", voucherCode);
        voucher.put("person_code", personCode);
        if (StringUtils.isBlank(remark)) {
            remark = "拒绝";
            if (isAgree) {
                remark = "同意";
            }
        }
        voucher.put("opinion", remark);
        voucher.put("agree", isAgree ? "1" : "0");  // 是否同意: 1=同意; 0=不同意
        JSONObject purchase = new JSONObject();
        purchase.put("purchaseorder", voucher);
        U8R r = createPOSTRequest("/api/purchaseorder/audit", param, purchase.toJSONString());
        log.info("响应数据: {}", r);
    }

    /**
     * 用户登录授权
     */
    @Override
    public void login2Authorization(String tradeId, String userId, String password) {
        Map param = initParamNeedToken();
        param.put("tradeid", tradeId);
        // 与原login接口不同之处在于，原login接口登录日期非当前系统日期，导致审核等相关操作的日期错误
        param.put("date", DateUtil.format(DateUtil.date(System.currentTimeMillis()), "yyyy-MM-dd"));
        JSONObject body = new JSONObject();
        body.put("user_id", userId);
        body.put("password", password);
        JSONObject user = new JSONObject();
        user.put("user", body);
        U8R r = createPOSTRequest("/api/user/login_v2", param, user.toJSONString());
        log.info("响应数据: {}", r);
    }
}